/***************************************************************************
 *   Copyright (C) 2004 by Antonio Fasolato                                *
 *   Antonio.Fasolato@poste.it                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "mainwidget.h"
#include "options.h"

#include <qlabel.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qbuffer.h>
#include <qprocess.h>
#include <qbuttongroup.h>
#include <qtabwidget.h>
#include <qcursor.h>
#include <qfile.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qimage.h>
#include <qdragobject.h>
#include <qscrollview.h>
#include <qwidgetstack.h>

#include <kurldrag.h>
#include <kfiledialog.h>
#include <kimageio.h>
#include <kio/netaccess.h>
#include <klocale.h>
#include <knuminput.h>
#include <kcombobox.h>
#include <kcolorbutton.h>
#include <kpushbutton.h>
#include <kdialogbase.h>
#include <kmessagebox.h>

#include <cstdlib>
#include <fstream>
#include <vector>
#include <utility>
#include <unistd.h>
using namespace std;

MainWidget::MainWidget(QWidget* parent, const char* name, WFlags fl)
    : MainWidgetBase(parent,name,fl), m_changed(false)
{
  KImageIO::registerFormats();

  setAcceptDrops(true);
  
  /*Scrolled image viewers*/
  delete imagesTabs->page(0);
  QScrollView *originalScroll=new QScrollView(this);
  QScrollView *previewScroll=new QScrollView(this);
  imagesTabs->addTab(originalScroll,i18n("Original image"));
  imagesTabs->addTab(previewScroll,i18n("Preview"));
  originalLabel=new QLabel(this);
  previewLabel=new QLabel(this);
  originalLabel->setGeometry(0,0,10,10);
  previewLabel->setGeometry(0,0,10,10);
  originalScroll->addChild(originalLabel);
  previewScroll->addChild(previewLabel);

  /*Load default options*/
  Options *o=new Options();
  o->defaultOptions();
  m_state.clear();
  m_oldPreviews.clear();
  m_state.push_back(o);
  m_oldPreviews.push_back(QPixmap());
  m_currentState=m_state.begin();
  m_currentPreview=m_oldPreviews.begin();

  connect(&tracer,SIGNAL(processExited()),this,SLOT(tracerExited()));
  connect(&tracer,SIGNAL(wroteToStdin()),this,SLOT(writeCompleted()));
  connect(traceBtn,SIGNAL(clicked()),this,SLOT(trace()));
  connect(defaultBtn,SIGNAL(clicked()),this,SLOT(defaultOptions()));

  readOptions();
}

MainWidget::~MainWidget()
{
  if (tracer.isRunning())
    tracer.tryTerminate();
  if(m_tmpFilename!="")
    deleteTmpFilename();
}

void MainWidget::fileOpen()
{
  KFileDialog d(QString::null,KImageIO::pattern(),this,i18n("Select an image"),true);
  d.exec();

  if(d.result()==QDialog::Accepted)
    getImageFromURL(d.selectedURL());
}

void MainWidget::fileSave()
{
  if(m_outputFilename=="" && m_changed)
  {
    QString filter;
    if((**m_currentState)["outputFormat"]=="eps")
      filter=i18n("*.eps|Encapsulated postscript");
    else if((**m_currentState)["outputFormat"]=="postscript")
      filter=i18n("*.ps|Postscript");
    else if((**m_currentState)["outputFormat"]=="SVG")
      filter=i18n("*.svg|SVG images");
    else if((**m_currentState)["outputFormat"]=="Gimp path")
      filter=i18n("*.*|Gimp path");
    else if((**m_currentState)["outputFormat"]=="Xfig")
      filter=i18n("*.fig|XFig drawing");
    else
      filter=i18n("*.pgm|PGM image");
    QString saveFilename=KFileDialog::getSaveFileName(QString::null,filter,this,i18n("Select the file to save to"));

    bool salva=false;
    while(saveFilename!="")
    {
      int res;
      if(QFile::exists(saveFilename))
        res=KMessageBox::warningYesNo(this, i18n("Overwrite ")+saveFilename+i18n("?"));
      else
      {
        salva=true;
        break;
      }
      if(res==KMessageBox::Yes)
      {
        salva=true;
        break;
      }
      else
        saveFilename=KFileDialog::getSaveFileName(QString::null,filter,this,i18n("Select the file to save to"));
    }

    if(salva)
    {
      m_outputFilename=saveFilename;
      ifstream in(m_tmpFilename);
      ofstream out(m_outputFilename);
      if(!in || !out)
      {
        emit signalChangeStatusbar(i18n(i18n("Error while saving file")));
        return;
      }
      emit signalChangeStatusbar(i18n("Saving ")+m_outputFilename+i18n("..."));
      char ch;
      while(in.get(ch))
        out.put(ch);
      in.close();
      out.close();
      m_changed=false;
      emit signalChangeStatusbar(i18n("File ")+m_outputFilename+i18n(" saved"));
    }
    else
      emit signalChangeStatusbar(i18n("File not saved"));
  }
  else if (m_changed)
  {
    ifstream in(m_tmpFilename);
    ofstream out(m_outputFilename);
    if(!in || !out)
    {
      emit signalChangeStatusbar(i18n("Error while saving file"));
      return;
    }
    emit signalChangeStatusbar(i18n("Saving ")+m_outputFilename+i18n("..."));
    char ch;
    while(in.get(ch))
      out.put(ch);

    in.close();
    out.close();
    emit signalChangeStatusbar(i18n("File ")+m_outputFilename+i18n(" saved"));
    m_changed=false;
  }
}

void MainWidget::fileSaveAs()
{
  if(previewLabel->pixmap()!=0)
  {
    QString filter;
    if((**m_currentState)["outputFormat"]=="eps")
      filter=i18n("*.eps|Encapsulated postscript");
    else if((**m_currentState)["outputFormat"]=="postscript")
      filter=i18n("*.ps|Postscript");
    else if((**m_currentState)["outputFormat"]=="SVG")
      filter=i18n("*.svg|SVG images");
    else if((**m_currentState)["outputFormat"]=="Gimp path")
      filter=i18n("*.*|Gimp path");
    else if((**m_currentState)["outputFormat"]=="Xfig")
      filter=i18n("*.fig|XFig drawing");
    else
      filter=i18n("*.pgm|PGM image");
    QString saveFilename=KFileDialog::getSaveFileName(QString::null,filter,this,i18n("Select the file to save to"));

    bool salva=false;
    while(saveFilename!="")
    {
      int res;
      if(QFile::exists(saveFilename))
        res=KMessageBox::warningYesNo(this,i18n("Overwrite ")+saveFilename+i18n("?"));
      else
      {
        salva=true;
        break;
      }
      if(res==KMessageBox::Yes)
      {
        salva=true;
        break;
      }
      else
        saveFilename=KFileDialog::getSaveFileName(QString::null,filter,this,i18n("Select the file to save to"));
    }

    if(salva)
    {
      m_outputFilename=saveFilename;
      ifstream in(m_tmpFilename);
      ofstream out(m_outputFilename);
      if(!in || !out)
      {
        emit signalChangeStatusbar(i18n("Error while saving file"));
        return;
      }
      emit signalChangeStatusbar(i18n("Saving ")+m_outputFilename+i18n("..."));
      char ch;
      while(in.get(ch))
        out.put(ch);
      in.close();
      out.close();
      m_changed=false;
      emit signalChangeStatusbar(i18n("File ")+m_outputFilename+i18n(" saved"));
    }
    else
      emit signalChangeStatusbar(i18n("File not saved"));
  }
}

void MainWidget::fileClose()
{
  originalLabel->clear();
  originalLabel->setGeometry(0,0,10,10);
  previewLabel->clear();
  previewLabel->setGeometry(0,0,10,10);
  if(m_tmpFilename!="" && !deleteTmpFilename())
    emit signalChangeStatusbar(i18n("Warning: could not remove temporary file."));
  Options *o=new Options();
  o->defaultOptions();
  m_state.clear();
  m_oldPreviews.clear();
  m_state.push_back(o);
  m_oldPreviews.push_back(QPixmap());
  m_currentState=m_state.begin();
  m_currentPreview=m_oldPreviews.begin();
  readOptions();
}

void MainWidget::cut()
{
  if(originalLabel->pixmap()!=0)
  {
    QApplication::clipboard()->setPixmap(QPixmap(*(originalLabel->pixmap())));
    fileClose();
  }
}

void MainWidget::copy()
{
  if(originalLabel->pixmap()!=0)
  {
    QApplication::clipboard()->setPixmap(QPixmap(*(originalLabel->pixmap())));
  }
}

void MainWidget::paste()
{
  QPixmap clip=QApplication::clipboard()->pixmap();
  if(!clip.isNull())
  {
    if(!createTmpFilename())
    {
      emit signalChangeStatusbar(i18n("Could not create temporary filename"));
      return;
    }

    m_outputFilename="";
	 QPixmap pix(QApplication::clipboard()->pixmap());
	 originalLabel->clear();
	 originalLabel->setGeometry(0,0,pix.width(),pix.height());
    originalLabel->setPixmap(pix);
    previewLabel->clear();
	 previewLabel->setGeometry(0,0,10,10);
    imagesTabs->setCurrentPage(0);
    emit signalChangeStatusbar(i18n("Image pasted."));

    Options *o=new Options();
    if (!preserveCheck->isChecked())
    {  //I have to reset the options
      o->defaultOptions();
    }
    else
    {
      o=*m_currentState;
    }
    m_state.clear();
    m_oldPreviews.clear();
    m_state.push_back(o);
    if(previewLabel->pixmap()!=0)
      m_oldPreviews.push_back(*(previewLabel->pixmap()));
    else
      m_oldPreviews.push_back(QPixmap());
    m_currentState=m_state.begin();
    m_currentPreview=m_oldPreviews.begin();

    readOptions();
  }
}

void MainWidget::getImageFromURL(KURL url)
{
  QString filename;				// prevent overwriting previous local file with current remote one
  if (!url.isLocalFile())
  {           // download to temporary file
    if (!KIO::NetAccess::download(url,filename,this))
      emit signalChangeStatusbar(i18n("Selected URL is not a valid image"));
    return;
  }
  else
    filename=url.path();

  if(!createTmpFilename())
  {
    emit signalChangeStatusbar(i18n("Could not create temporary filename"));
    return;
  }

  m_outputFilename="";
  QPixmap pix(filename);
  originalLabel->clear();
  originalLabel->setGeometry(0,0,pix.width(),pix.height());
  originalLabel->setPixmap(pix);
  previewLabel->clear();
  previewLabel->setGeometry(0,0,10,10);
  imagesTabs->setCurrentPage(0);
  emit signalChangeStatusbar(i18n("file ")+filename+i18n(" loaded."));

  if (!url.isLocalFile())
    KIO::NetAccess::removeTempFile(filename);

  Options *o=new Options();
  if (!preserveCheck->isChecked())
  {  //I have to reset the options
    o->defaultOptions();
  }
  else
  {
    o=*m_currentState;
  }
  m_state.clear();
  m_oldPreviews.clear();
  m_state.push_back(o);
  if(previewLabel->pixmap()!=0)
    m_oldPreviews.push_back(*(previewLabel->pixmap()));
  else
    m_oldPreviews.push_back(QPixmap());
  m_currentState=m_state.begin();
  m_currentPreview=m_oldPreviews.begin();

  readOptions();
}

void MainWidget::readOptions()
{
  //input
  blackLevel->setValue(((**m_currentState)["blackLevel"]).toFloat());
  invertInput->setChecked((**m_currentState)["invertInput"]=="y");

  //output
  outputFormat->setCurrentText((**m_currentState)["outputFormat"]);
  if((**m_currentState)["outputFormat"]=="eps" || (**m_currentState)["outputFormat"]=="postscript")
  {       //eps format
    pageSize->setCurrentText((**m_currentState)["pageSize"]);
    optimizedNumericalCode->setChecked((**m_currentState)["optimizedNumericalCode"]=="y");
    level1->setChecked((**m_currentState)["compressionLevel"]=="1");
    level2->setChecked((**m_currentState)["compressionLevel"]=="2");
    level3->setChecked((**m_currentState)["compressionLevel"]=="3");
  }
  else if((**m_currentState)["outputFormat"]=="SVG")            //SVG format
    SVGGroup->setChecked((**m_currentState)["SVGGroup"]=="y");
  else if((**m_currentState)["outputFormat"]=="Gimp path");          //Gimp format
  else if((**m_currentState)["outputFormat"]=="Xfig path");          //Xfig format
  else
  {                                                              //pgm format
    antialiasGamma->setValue(((**m_currentState)["antialiasGamma"]).toFloat());
    //Scale
    if(((**m_currentState)["scale"]).find('x')==-1)
    {   //only one value
      scaleAll->setValue(((**m_currentState)["scale"]).toFloat());
      scaleAllBTN->setChecked(true);
    }
    else
    {
      QString tmp=(**m_currentState)["scale"];
      scaleX->setValue((tmp.section('x',0,0)).toFloat());
      scaleY->setValue((tmp.section('x',1,1)).toFloat());
      scaleXYBTN->setChecked(true);
    }
  }

  //Color
  foregroundBtn->setColor(QColor((**m_currentState)["foregroundBtn"]));
  opaque->setChecked((**m_currentState)["opaque"]=="y");
  backgroundBtn->setColor(QColor((**m_currentState)["backgroundBtn"]));

  //Algorithm
  policy->setCurrentText((**m_currentState)["policy"]);
  despeckle->setValue(((**m_currentState)["despeckle"]).toInt());
  cornerThreshold->setValue(((**m_currentState)["cornerThreshold"]).toFloat());
  optimizationTolerance->setValue(((**m_currentState)["optimizationTolerance"]).toFloat());
  outputQuantization->setValue(((**m_currentState)["outputQuantization"]).toInt());
  curveOptimization->setChecked((**m_currentState)["curveOptimization"]=="y");

  //Transformation
  width->setValue(((**m_currentState)["width"]).toFloat());
  height->setValue(((**m_currentState)["height"]).toFloat());
  stretch->setValue(((**m_currentState)["stretch"]).toFloat());
  rotation->setValue(((**m_currentState)["rotation"]).toFloat());

  //Resolution
  if(((**m_currentState)["resolution"]).find('x')==-1)
  { //only one value
    resolutionAll->setValue(((**m_currentState)["resolution"]).toFloat());
    resolutionAllBTN->setChecked(true);
  }
  else
  {
    QString tmp=(**m_currentState)["resolution"];
    resolutionX->setValue((tmp.section('x',0,0)).toFloat());
    resolutionY->setValue((tmp.section('x',1,1)).toFloat());
    resolutionXYBTN->setChecked(true);
  }

  //Margins
  if(((**m_currentState)["margins"]).find('x')==-1)
  {   //only one value
    marginsAll->setValue(((**m_currentState)["margins"]).toFloat());
    marginsAllBTN->setChecked(true);
  }
  else
  {
    QString tmp=(**m_currentState)["margins"];
    marginsL->setValue((tmp.section('x',0,0)).toFloat());
    marginsR->setValue((tmp.section('x',1,1)).toFloat());
    marginsT->setValue((tmp.section('x',2,2)).toFloat());
    marginsB->setValue((tmp.section('x',3,3)).toFloat());
    marginsXYBTN->setChecked(true);
  }
}

void MainWidget::getOptions()
{
  //input-output
  (*m_currentState)->setValue("blackLevel",QString::number(blackLevel->value()));
  if(invertInput->isChecked())
    (*m_currentState)->setValue("invertInput","y");
  else
    (*m_currentState)->setValue("invertInput","n");
  (*m_currentState)->setValue("outputFormat",outputFormat->currentText());
  if(outputFormat->currentText()=="eps" || outputFormat->currentText()=="postscript")
  {  //ps options
    (*m_currentState)->setValue("pageSize",pageSize->currentText());
    (*m_currentState)->setValue("optimizedNumericalCode",(optimizedNumericalCode->isChecked())?"y":"n");
    QButton *b=compressionLevel->selected();
    if(b->text()=="None")
      (*m_currentState)->setValue("compressionLevel","1");
    else if(b->text()=="Level 2")
      (*m_currentState)->setValue("compressionLevel","2");
    else
      (*m_currentState)->setValue("compressionLevel","3");
  }
  else if(outputFormat->currentText()=="SVG")
  {  //SVG options
    (*m_currentState)->setValue("SVGGroup",(SVGGroup->isChecked())?"y":"n");
  }
  else if(outputFormat->currentText()=="Gimp path")
  {  //Gimp options
  }
  else if(outputFormat->currentText()=="Xfig")
  {  //Xfig options
  }
  else
  {  //pgm options
    (*m_currentState)->setValue("antialiasGamma",QString::number(antialiasGamma->value()));
    //Scale
    if(scaleAll->isEnabled())
      (*m_currentState)->setValue("scale",QString::number(scaleAll->value()));
    else
      (*m_currentState)->setValue("scale",QString::number(scaleX->value())+"x"+QString::number(scaleY->value()));
  }

  //Color
  QColor c=foregroundBtn->color();
  (*m_currentState)->setValue("foregroundBtn",c.name());
  (*m_currentState)->setValue("opaque",(opaque->isChecked())?"y":"n");
  c=backgroundBtn->color();
  (*m_currentState)->setValue("backgroundBtn",c.name());

  //Algorithm
  (*m_currentState)->setValue("policy",policy->currentText());
  (*m_currentState)->setValue("despeckle",QString::number(despeckle->value()));
  (*m_currentState)->setValue("cornerThreshold",QString::number(cornerThreshold->value()));
  (*m_currentState)->setValue("optimizationTolerance",QString::number(optimizationTolerance->value()));
  (*m_currentState)->setValue("outputQuantization",QString::number(outputQuantization->value()));
  (*m_currentState)->setValue("curveOptimization",(curveOptimization->isChecked())?"y":"n");

  //Transformation
  (*m_currentState)->setValue("width",QString::number(width->value()));
  (*m_currentState)->setValue("height",QString::number(height->value()));
  (*m_currentState)->setValue("stretch",QString::number(stretch->value()));
  (*m_currentState)->setValue("rotation",QString::number(rotation->value()));

  //Resolution
  if(resolutionAll->isEnabled())
    (*m_currentState)->setValue("resolution",QString::number(resolutionAll->value()));
  else
    (*m_currentState)->setValue("resolution",QString::number(resolutionX->value())+"x"+QString::number(resolutionY->value()));

  //Margins
  if(marginsAll->isEnabled())
    (*m_currentState)->setValue("margins",QString::number(marginsAll->value()));
  else
    (*m_currentState)->setValue("margins",QString::number(marginsL->value())+"x"+QString::number(marginsR->value())+"x"+ QString::number(marginsT->value())+"x"+QString::number(marginsB->value()));
}

QStringList MainWidget::createCommandLine()
{
  QStringList toReturn;
  toReturn+="potrace";

  //input
  if((**m_currentState)["blackLevel"]!="0.5")   //only if it is not the default value
    toReturn+="-k"+(**m_currentState)["blackLevel"];
  if((**m_currentState)["invertInput"]=="y")
    toReturn+="-i";

  //output
  toReturn+="-o";
  toReturn+=m_tmpFilename;
  if((**m_currentState)["outputFormat"]=="eps")
  {                   //eps format
    toReturn+="-e";
    toReturn+="-P"+(**m_currentState)["pageSize"];
    toReturn+="-C"+(**m_currentState)["foregroundBtn"];
    if((**m_currentState)["opaque"]=="y")
    {
      toReturn+="--opaque";
      //toReturn+="--fillcolor"+(**m_currentState)["backgroundBtn"];
    }
    if((**m_currentState)["optimizedNumericalCode"]=="n")
      toReturn+="-q";
    if((**m_currentState)["compressionLevel"]=="1")
      toReturn+="-c";
    if((**m_currentState)["compressionLevel"]=="2")
      toReturn+="-2";
    if((**m_currentState)["compressionLevel"]=="3")
      toReturn+="-3";
  }
  else if((**m_currentState)["outputFormat"]=="postscript")
  {      //ps format
    toReturn+="-p";
    toReturn+="-P"+(**m_currentState)["pageSize"];
    toReturn+="-C"+(**m_currentState)["foregroundBtn"];
    if((**m_currentState)["opaque"]=="y")
    {
      toReturn+="--opaque";
      //toReturn+="--fillcolor"+(**m_currentState)["backgroundBtn"];
    }
    if((**m_currentState)["optimizedNumericalCode"]=="n")
      toReturn+="-q";
    if((**m_currentState)["compressionLevel"]=="1")
      toReturn+="-c";
    if((**m_currentState)["compressionLevel"]=="2")
      toReturn+="-2";
    if((**m_currentState)["compressionLevel"]=="3")
      toReturn+="-3";
  }
  else if((**m_currentState)["outputFormat"]=="SVG")
  {            //SVG format
    toReturn+="-s";
    toReturn+="-C"+(**m_currentState)["foregroundBtn"];
    if((**m_currentState)["opaque"]=="y")
    {
      toReturn+="--opaque";
      //toReturn+="--fillcolor"+(**m_currentState)["backgroundBtn"];
    }
    if((**m_currentState)["SVGGroup"]=="y")
      toReturn+="--group";
  }
  else if((**m_currentState)["outputFormat"]=="Gimp path")
  {            //Gimp format
    toReturn+="-bgimppath";
  }
  else if((**m_currentState)["outputFormat"]=="Xfig")
  {            //Xfig format
    toReturn+="-bxfig";
    toReturn+="-C"+(**m_currentState)["foregroundBtn"];
    if((**m_currentState)["opaque"]=="y")
    {
      toReturn+="--opaque";
      //toReturn+="--fillcolor"+(**m_currentState)["backgroundBtn"];
    }
  }
  else
  {                                                               //pgm format
    toReturn+="-g";
    toReturn+="-G"+(**m_currentState)["antialiasGamma"];
    if((**m_currentState)["opaque"]=="y")
    {
      toReturn+="--opaque";
      //toReturn+="--fillcolor"+(**m_currentState)["backgroundBtn"];
    }
  }

  //Algorithm
  toReturn+="-z"+(**m_currentState)["policy"];
  toReturn+="-t"+(**m_currentState)["despeckle"];
  toReturn+="-a"+(**m_currentState)["cornerThreshold"];
  toReturn+="-O"+(**m_currentState)["optimizationTolerance"];
  toReturn+="-u"+(**m_currentState)["outputQuantization"];
  if((**m_currentState)["curveOptimization"]=="n")
    toReturn+="-n";

  //Transformation
  if((**m_currentState)["width"]!="0")
    toReturn+="-W"+(**m_currentState)["width"];
  if((**m_currentState)["height"]!="0")
    toReturn+="-H"+(**m_currentState)["height"];
  if((**m_currentState)["stretch"]!="1")
    toReturn+="-S"+(**m_currentState)["stretch"];
  if((**m_currentState)["rotation"]!="0")
    toReturn+="-A"+(**m_currentState)["rotation"];

  //Resolution
  if((**m_currentState)["resolution"]!="0" && (**m_currentState)["resolution"]!="0x0")
    toReturn+="-r"+(**m_currentState)["resolution"];

  if((**m_currentState)["margins"]!="0" && (**m_currentState)["margins"]!="0x0x0x0")
  {
    if((**m_currentState)["margins"].find('x')==-1)
      toReturn+="-M"+(**m_currentState)["margins"];
    else
    {
      QString tmp=(**m_currentState)["margins"];
      toReturn+="-L"+tmp.section('x',0,0);
      toReturn+="-R"+tmp.section('x',1,1);
      toReturn+="-T"+tmp.section('x',2,2);
      toReturn+="-B"+tmp.section('x',3,3);
    }
  }

  return toReturn;
}

void MainWidget::trace()
{
  if(originalLabel->pixmap()==0)
  {
    emit(signalChangeStatusbar(i18n("You must open an image first.")));
    return;
  }
  setCursor(WaitCursor);
  if (tracer.isRunning())
  {       // already tracing, this means abort
    aborted=true;
    tracer.tryTerminate();
    return;
  }
  aborted=false;

  getOptions();
  QStringList commandLine=createCommandLine();
  tracer.clearArguments();
  tracer.setArguments(commandLine);

  if(!(tracer.start()))
  {
    previewLabel->setText(i18n("The process couldn't be started\nThe process executed was:\n")+commandLine.join(" "));
    emit(signalChangeStatusbar("Error executing command"));
    setCursor(ArrowCursor);
    imagesTabs->setCurrentPage(1);
  }
  else
  {
    emit(signalChangeStatusbar(i18n("Tracing...")));
    traceBtn->setText(i18n("Abort"));
    QByteArray pixData;
    QBuffer buf(pixData);
    buf.open(IO_WriteOnly);
    originalLabel->pixmap()->save(&buf,"PPM");
    tracer.writeToStdin(pixData);
  }
}

void MainWidget::tracerExited()
{
  traceBtn->setText(i18n("Trace!"));
  if (aborted)
  {
    emit(signalChangeStatusbar(i18n("Aborted")));
    setCursor(ArrowCursor);
    return;
  }
  if(tracer.exitStatus()!=0)
  {
    emit(signalChangeStatusbar(i18n("Finished with errors")));
    QString tmp(i18n("Error during the process:"));
    tmp+=i18n("\nStandard output:\n");
    QString t=tracer.readLineStdout();
    while(t!=QString::null)
    {
      tmp+= t+"\n";
      t=tracer.readLineStdout();
    }
    tmp+=i18n("\nStandard error:\n");
    t=tracer.readLineStderr();
    while(t!=QString::null)
    {
      tmp+=t+"\n";
      t=tracer.readLineStderr();
    }
    QStringList l=tracer.arguments();
    tmp+=i18n("\nExecuted:\n")+l.join(" ");
    previewLabel->setText(tmp);
    setCursor(ArrowCursor);
    imagesTabs->setCurrentPage(1);
    return;
  }

  if(autoTraceCheck->isChecked())
  {
    emit(signalChangeStatusbar(i18n("Loading preview image...")));
    QImage img(m_tmpFilename);
    QPixmap pix;
    if(pix.convertFromImage(img,0))
    {
      previewLabel->clear();
		previewLabel->setGeometry(0,0,pix.width(),pix.height());
      previewLabel->setPixmap(pix);
    }
    else
    {
      emit(signalChangeStatusbar(i18n("Output format not recognized")));
      previewLabel->setText(i18n("Sorry, but the image format which you are working with\nis not recognized by KDE.\nThe image will be correctly saved if you want to,\nbut to see a preview try working with another image format (EPS and PGM showld work well)\nand only when you are satisfied with the result revert to the format\nof your choiche and save the result."));
      imagesTabs->setCurrentPage(1);
      setCursor(ArrowCursor);
      return;
    }

    imagesTabs->setCurrentPage(1);
  }

  setCursor(ArrowCursor);
  m_changed=true;
  emit(signalChangeStatusbar(i18n("Image traced")));
}

void MainWidget::writeCompleted()
{
  tracer.closeStdin();
}

bool MainWidget::createTmpFilename()
{
  char tmp[]="/tmp/potraceguiTmp-XXXXXX";
  if (mkstemp(tmp)!=-1)
  {
    m_tmpFilename=QString(tmp);
    return true;
  }
  else
    return false;
}

bool MainWidget::deleteTmpFilename()
{
  QFile tmp(m_tmpFilename);
  bool toReturn=tmp.remove();
  if(toReturn==true)
    m_tmpFilename="";
  return toReturn;
}

int MainWidget::notificationDialog(QString caption, QString text)
{
  KDialogBase w(this, "potracegui", true, caption, KDialogBase::Ok|KDialogBase::Cancel,KDialogBase::Ok,true);
  w.setMainWidget(new QLabel(text,&w));
  w.exec();
  return w.result();
}

void MainWidget::dragEnterEvent(QDragEnterEvent* event)
{
  event->accept(KURLDrag::canDecode(event));
}

void MainWidget::dropEvent(QDropEvent* event)
{
  KURL::List urls;
  if (KURLDrag::decode(event,urls))
  {
    getImageFromURL(urls[0]);
  }
}

void MainWidget::defaultOptions()
{
  /*Load default options*/
  Options *o=new Options();
  o->defaultOptions();
  m_state.clear();
  m_oldPreviews.clear();
  m_state.push_back(o);
  m_oldPreviews.push_back(QPixmap());
  m_currentState=m_state.begin();
  m_currentPreview=m_oldPreviews.begin();

  readOptions();
  optionsStack->raiseWidget(psPage);
}

void MainWidget::showOptions(const QString &selection) {
	if(selection=="SVG")
		optionsStack->raiseWidget(svgPage);
	else if(selection=="pgm")
		optionsStack->raiseWidget(pgmPage);
	else if(selection=="postscript" || selection=="eps")
		optionsStack->raiseWidget(psPage);
	else
		optionsStack->raiseWidget(undefinedPage);
}

#include "mainwidget.moc"
